module delay(
    input logic [7:0] wave,
    input logic fast_clk,
    input logic main_clk,
    input logic clk_cycle,
    input logic rst,
    output logic [8:0] out_wave,
    output logic delay_fetch
    );
    
    reg [1:0] state;
    
    always@(state) begin 
        case (state)
            2'b00: begin
               out_wave = 8'h00;
               delay_fetch = 1'b0;
               end
            2'b01: begin
               out_wave = wave;
               delay_fetch = 1'b1;
               end
            2'b10: begin
               out_wave = out_wave;
               delay_fetch = 1'b0;
               end
            default: begin
               out_wave = 8'h00;
               delay_fetch = 1'b0;
               end
        endcase
    end
    
    always @ (posedge fast_clk) begin
        if (rst)
            state <= 2'b00;
        else
        case (state)
           2'b00:
              state <= 2'b01;
           2'b01:
              state <= 2'b10;
           2'b10:
              if (clk_cycle==0)
                 state <= 2'b10;
              else
                 state <= 2'b01;
        endcase
        end
endmodule

module loop(
    input logic fast_clk,
    input logic[7:0] loop_addr,
    input logic[7:0] loop_repeat,
    input logic increase,
    input logic rst,
    output logic loop_fetch,
    output logic[7:0] L_jump);
    
    reg [1:0] state;
    reg [7:0] count;

    always @(fast_clk) begin
        if(rst)
            state <= 2'b00;
        else begin
        case(state) 
        2'b00: 
            state <= 2'b01;
        2'b01: begin
            if(increase) begin
                if(count < loop_repeat)
                    state <= 2'b10;
                else
                    state <= 2'b11;
            end else
                state <= 2'b01;
            end
        2'b10:
            state <= 2'b01;
        2'b11:
            state <= 2'b01;
        endcase
        end
    end
    
    always @(state) begin
        case(state)
        2'b00: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b01: begin
            count <= count;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b10: begin
            count <= count + 1'b1;
            L_jump <= loop_addr;
            loop_fetch <= 1'b1;
            end
        2'b11: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b1;
            end
        endcase
    end
    
endmodule

