module data_handle(
    input logic rst,
    input logic fast_clk,
    input logic[7:0] data,
    output logic[3:0] wave,
    output logic change,
    output logic[7:0] loop_addr,
    output logic[7:0] loop_repeat,
    output logic increase,
    output logic jump_fetch,
    output logic[7:0] D_jump
    );
    
     reg [3:0] state;
     reg [3:0] addr_part1;
    
    always_comb begin
        if(rst)
            state = 4'h0;
        else
            state = data[7:4];
    end
    
    always@(posedge fast_clk) begin 
        case (state)
        4'h0: begin
            wave = 4'h0;
            change = 1'b0;
            loop_addr = 8'h00;
            loop_repeat = 8'h00;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = 8'h00;
            end 
        4'h2: begin
            wave = data[3:0];
            change = 1'b1;
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h3: begin 
            wave = data[3:0];
            change = 1'b0;
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h4: begin
            addr_part1 = data[3:0];
            loop_repeat = loop_repeat;
            wave = wave;
            change = 1'b0;
            loop_addr = loop_addr;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h5: begin 
            addr_part1 = addr_part1;
            loop_repeat = {data[3:0], addr_part1};
            wave = wave;
            change = 1'b0;
            loop_addr = loop_addr;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h6: begin
            addr_part1 = data[3:0];
            loop_addr = loop_addr;
            wave = wave;
            change = 1'b0;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = D_jump;
            end 
         4'h7: begin 
            addr_part1 = addr_part1;
            loop_addr = {data[3:0], addr_part1};
            wave = wave;
            change = 1'b0;
            loop_repeat = loop_repeat;
            increase = 1'b1;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
         4'h8: begin
            wave = wave;
            change = 1'b0;
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = D_jump;
            addr_part1 = data[3:0];
            end 
         4'h9: begin
            wave = wave;
            change = 1'b0;
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = {D_jump, addr_part1};
            addr_part1 = addr_part1;
            end 
         4'h10: begin
            wave = wave;
            change = 1'b0;
            loop_addr = 8'h00;
            loop_repeat = 8'h00;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = D_jump;
            addr_part1 = data[3:0];
            end 
         4'h11: begin
            wave = wave;
            change = 1'b0;
            loop_addr = 8'h00;
            loop_repeat = 8'h00;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = {D_jump, addr_part1};
            addr_part1 = addr_part1;
            end 
         default: begin
            wave = 4'h0;
            change = 1'b0;
            loop_addr = 8'h00;
            loop_repeat = 8'h00;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = 8'h00;
            end 
        endcase
    end
endmodule
