module delay(
    input logic [7:0] wave,
    input logic [3:0] wave_pos,
    input logic fast_clk,
    input logic main_clk,
    input logic clk_cycle,
    input logic rst,
    output logic [8:0] out_wave,
    output logic delay_fetch
    );
    
   reg [1:0] state;
   reg [3:0] count;
   reg [3:0] position;
    
    always@(state) begin 
        case (state)
            2'b00: begin
               out_wave = 8'h00;
               delay_fetch = 1'b0;
               count = 4'h0;
               position = 4'h0;
               end
            2'b01: begin
               out_wave = wave;
               delay_fetch = 1'b1;
               count <= 4'b0;
               position <= wave_pos;
               end
            2'b10: begin
               out_wave = out_wave;
               delay_fetch = 1'b0;
               count <= count + 1'b1;
               position <= position;
               end
            2'b11: begin
               out_wave = out_wave;
               delay_fetch = 1'b0;
               count <= count;
               position <= position;
               end
            default: begin
               out_wave = 8'h00;
               delay_fetch = 1'b0;
               count <= 4'h0;
               position <= 4'h0;
               end
        endcase
    end
    
    always @ (posedge fast_clk) begin
        if (rst)
            state <= 2'b00;
        else
        case (state)
           2'b00:
              state <= 2'b01;
           2'b01:
              state <= 2'b10;
           2'b10:
              state <= 2'b11;
           2'b11:
              if (clk_cycle) begin
                if(count == position)
                    state <= 2'b01;
                else
                    state <= 2'b10;
              end
              else
                 state <= 2'b11;
        endcase
        end
endmodule

module wave_assemble (
    input logic [3:0] data,
    input logic change,
    input logic fast_clk,
    input logic rst,
    output logic assemble_fetch,
    output logic [7:0] wave);
    
    reg [7:0] temp_wave;
    reg [2:0] state;
    
    always@(posedge fast_clk) begin
        if(rst)
            state <= 3'b000;
        else begin
        case(state)
        3'b000:
            state <= 3'b001;
        3'b001: begin
            if(change)
                if(data[3:3])
                    state <= 3'b010;
                else
                    state <= 3'b011;
            else
                state <= 3'b001;
            end
        3'b010: begin
            if(data[3:3])
                state <= 3'b010;
            else
                state <= 3'b011;
            end
        3'b011: 
            state <= 3'b100;
        3'b100:
            if(change)
                state <= 3'b100;
            else
                state <= 3'b001;
        endcase
        end
    end
    
    
    always@(state) begin
        case(state)
        3'b000: begin
            wave <= 8'b00;
            temp_wave <= 8'b00;
            assemble_fetch <= 1'b0;
            end
        3'b001: begin
            wave <= wave;
            temp_wave <= wave;
            assemble_fetch <= 1'b0;
            end
        3'b010: begin 
            wave <= wave;
            temp_wave <= (temp_wave ^ (1'b1 << data));
            assemble_fetch <= 1'b1;
            end
        3'b011: begin
            wave <= (temp_wave ^ (1'b1 << data));
            temp_wave <= temp_wave;
            assemble_fetch <= 1'b0;
            end
        3'b100: begin
            wave <= wave;
            temp_wave <= wave;
            assemble_fetch <= 1'b0;
            end
        endcase
    end
endmodule


module loop(
    input logic fast_clk,
    input logic[7:0] loop_addr,
    input logic[7:0] loop_repeat,
    input logic increase,
    input logic rst,
    output logic loop_fetch,
    output logic[7:0] L_jump);
    
    reg [1:0] state;
    reg [7:0] count;

    always @(fast_clk) begin
        if(rst)
            state <= 2'b00;
        else begin
        case(state) 
        2'b00: 
            state <= 2'b01;
        2'b01: begin
            if(increase) begin
                if(count < loop_repeat)
                    state <= 2'b10;
                else
                    state <= 2'b11;
            end else
                state <= 2'b01;
            end
        2'b10:
            state <= 2'b01;
        2'b11:
            state <= 2'b01;
        endcase
        end
    end
    
    always @(state) begin
        case(state)
        2'b00: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b01: begin
            count <= count;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b10: begin
            count <= count + 1'b1;
            L_jump <= loop_addr;
            loop_fetch <= 1'b1;
            end
        2'b11: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b1;
            end
        endcase
    end
    
endmodule
