module top(
    input logic clk_in,
    input logic rst,
    input logic [1:0] command,
    input logic [7:0] data_in,
    output logic [7:0] data_out,
    output logic[7:0] outwave 
    );
    
    // all wires to connect the modules 
    logic rst_all;
    logic main_clk;
    logic fast_clk;
    logic clk_cycle;
    logic locked;
    //addresses in memory 
    logic[7:0] nxtAddr;
    logic[7:0] D_jump;
    logic[7:0] L_jump;
    //enable signals
    logic loop_fetch;
    logic delay_fetch;
    logic assemble_fetch;
    logic jump_fetch;
    logic enable;
    //Delay input
    logic[7:0] wave;
    logic change;
    logic[3:0] change_bit;
    logic[3:0] wave_pos;
    //loop input
    logic[7:0] loop_addr;
    logic[7:0] loop_repeat;
    logic increase;
    logic increase_edge;
    //memory 
    logic wr;
    logic rd;
    logic [7:0] wr_data;
    logic [7:0] rd_data;
    logic [11:0] data;
    logic [7:0] wr_addr;
    
    
    clk_module clk1 (.main_clk(main_clk), .fast_clk(fast_clk), .reset(rst), .locked(locked), .clk_in1(clk_in));
    //nxt_posedge edge_detector_clk (.fast_clk(fast_clk), .signal(main_clk), .rst(rst_all), .clk_cycle(clk_cycle));
    clock_counter edge_detector_clk (.fast_clk(fast_clk), .rst(rst_all), .clk_edge(clk_cycle));
    interfaceA interface1 (.fast_clk(fast_clk), .command(command), .data_in(data_in), .rd_data(rd_data), .addr(wr_addr), .wr(wr), .wr_data(wr_data), .rd(rd), .data_out(data_out));
    init_modules init_module1 (.locked(locked), .rst_all(rst_all));
    enableCtrl enableCtrl1(.fast_clk(fast_clk), .delay_fetch(delay_fetch), .assemble_fetch(assemble_fetch), .loop_fetch(loop_fetch), .jump_fetch(jump_fetch), .enable(enable));
    nextAddress nextAddress1(.rst_all(rst_all), .fast_clk(fast_clk), .D_jump(D_jump), .L_jump(L_jump), .fetchEnable(enable), .nxt_addr(nxtAddr));
    memory memory1(.fast_clk(fast_clk), .wr(wr), .rd(rd), .wr_data(wr_data), .rd_data(rd_data), .addr(nxtAddr), .data(data));
    data_handle data_handle1(.rst(rst_all), .fast_clk(fast_clk), .data(data), .wave(change_bit), .change(change), .wave_pos(wave_pos), .loop_addr(loop_addr), .loop_repeat(loop_repeat), .increase(increase), .jump_fetch(jump_fetch), .D_jump(D_jump));
    wave_assemble wave_assemble1(.fast_clk(fast_clk), .rst(rst_all), .data(change_bit), .change(change), .wave(wave), .assemble_fetch(assemble_fetch));
    delay delay1(.wave(wave), .wave_pos(wave_pos), .main_clk(main_clk), .fast_clk(fast_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(outwave), .delay_fetch(delay_fetch));
    nxt_posedge edge_detector_loop (.fast_clk(fast_clk), .signal(increase), .rst(rst_all), .clk_cycle(increase_edge));
    loop loop1(.fast_clk(fast_clk), .loop_addr(loop_addr), .loop_repeat(loop_repeat), .increase(increase_edge), .rst(rst_all), .loop_fetch(loop_fetch), .L_jump(L_jump));
    
    
endmodule
