module delay(
    input logic [3:0] wave_pos,
    input logic fast_clk,
    input logic main_clk,
    input logic clk_cycle,
    input logic rst,
    output logic out_wave,
    output logic delay_fetch
    );
    
   reg [1:0] state;
   reg [3:0] count;
   reg [3:0] position;
    
    always@(state) begin 
        case (state)
            2'b00: begin
               out_wave = 1'b0;
               delay_fetch = 1'b0;
               count = 4'h0;
               position = 4'h0;
               end
            2'b01: begin
               out_wave = ~out_wave;
               delay_fetch = 1'b1;
               count <= 4'b0;
               position <= wave_pos;
               end
            2'b10: begin
               out_wave = out_wave;
               delay_fetch = 1'b0;
               count <= count + 1'b1;
               position <= position;
               end
            2'b11: begin
               out_wave = out_wave;
               delay_fetch = 1'b0;
               count <= count;
               position <= position;
               end
            default: begin
               out_wave = 1'b0;
               delay_fetch = 1'b0;
               count <= 4'h0;
               position <= 4'h0;
               end
        endcase
    end
    
    always @ (posedge fast_clk) begin
        if (rst)
            state <= 2'b00;
        else
        case (state)
           2'b00:
              state <= 2'b01;
           2'b01:
              state <= 2'b10;
           2'b10:
              state <= 2'b11;
           2'b11:
              if (clk_cycle) begin
                if(count == position)
                    state <= 2'b01;
                else
                    state <= 2'b10;
              end
              else
                 state <= 2'b11;
        endcase
        end
endmodule

module output_assembly(
    input logic signal0,
    input logic signal1,
    input logic signal2,
    input logic signal3,
    input logic signal4,
    input logic signal5,
    input logic signal6,
    input logic signal7,
    output logic[7:0] out_wave);
    
    always_comb begin
        out_wave[0] = signal0;
        out_wave[1] = signal1;
        out_wave[2] = signal2;
        out_wave[3] = signal3;
        out_wave[4] = signal4;
        out_wave[5] = signal5;
        out_wave[6] = signal6;
        out_wave[7] = signal7;
    end
endmodule


module loop(
    input logic fast_clk,
    input logic[7:0] loop_addr,
    input logic[7:0] loop_repeat,
    input logic increase,
    input logic rst,
    output logic loop_fetch,
    output logic[7:0] L_jump);
    
    reg [1:0] state;
    reg [7:0] count;

    always @(fast_clk) begin
        if(rst)
            state <= 2'b00;
        else begin
        case(state) 
        2'b00: 
            state <= 2'b01;
        2'b01: begin
            if(increase) begin
                if(count < loop_repeat)
                    state <= 2'b10;
                else
                    state <= 2'b11;
            end else
                state <= 2'b01;
            end
        2'b10:
            state <= 2'b01;
        2'b11:
            state <= 2'b01;
        endcase
        end
    end
    
    always @(state) begin
        case(state)
        2'b00: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b01: begin
            count <= count;
            L_jump <= 8'h00;
            loop_fetch <= 1'b0;
            end
        2'b10: begin
            count <= count + 1'b1;
            L_jump <= loop_addr;
            loop_fetch <= 1'b1;
            end
        2'b11: begin
            count <= 8'h01;
            L_jump <= 8'h00;
            loop_fetch <= 1'b1;
            end
        endcase
    end
    
endmodule
