module top(
    input logic clk_in,
    input logic rst,
    input logic [1:0] command,
    input logic [7:0] data_in,
    output logic [7:0] data_out,
    output logic[7:0] outwave 
    );
    
    // all wires to connect the modules 
    logic rst_all;
    logic main_clk;
    logic fast_clk;
    logic clk_cycle;
    logic locked;
    //addresses in memory 
    logic[7:0] nxtAddr;
    logic[7:0] D_jump;
    logic[7:0] L_jump;
    //enable signals
    logic loop_fetch;
    logic delay_fetch;
    logic assemble_fetch;
    logic jump_fetch;
    logic enable;
    //Delay input
    logic[7:0] wave;
    logic[3:0] toggle_pos;
    logic[3:0] wave_pos;
    logic sig0;
    logic sig1;
    logic sig2;
    logic sig3;
    logic sig4;
    logic sig5;
    logic sig6;
    logic sig7;
    //loop input
    logic[7:0] loop_addr;
    logic[7:0] loop_repeat;
    logic increase;
    logic increase_edge;
    //memory 
    logic wr;
    logic rd;
    logic [7:0] wr_data;
    logic [7:0] rd_data;
    logic [11:0] data;
    logic [7:0] wr_addr;
    logic [11:0] mem_sig1;
    logic [11:0] mem_sig2;
    logic [11:0] mem_sig3;
    logic [11:0] mem_sig4;
    logic [11:0] mem_sig5;
    logic [11:0] mem_sig6;
    logic [11:0] mem_sig7;
    
    
    
    clk_module clk1 (.main_clk(main_clk), .fast_clk(fast_clk), .reset(rst), .locked(locked), .clk_in1(clk_in));
    nxt_posedge edge_detector_clk (.fast_clk(fast_clk), .signal(main_clk), .rst(rst_all), .clk_cycle(clk_cycle));
    interfaceA interface1 (.fast_clk(fast_clk), .command(command), .data_in(data_in), .rd_data(rd_data), .addr(wr_addr), .wr(wr), .wr_data(wr_data), .rd(rd), .data_out(data_out));
    init_modules init_module1 (.locked(locked), .rst_all(rst_all));
    enableCtrl enableCtrl1(.delay_fetch(delay_fetch), .loop_fetch(loop_fetch), .jump_fetch(jump_fetch), .enable(enable));
    nextAddress nextAddress1(.rst_all(rst_all), .fast_clk(fast_clk), .D_jump(D_jump), .L_jump(L_jump), .fetchEnable(enable), .nxt_addr(nxtAddr));
    memory memory1(.fast_clk(fast_clk), .wr(wr), .rd(rd), .wr_data(wr_data), .rd_data(rd_data), .addr(nxtAddr), .data(data), .sig1(mem_sig1), .sig2(mem_sig2), .sig3(mem_sig3), .sig4(mem_sig4), .sig5(mem_sig5), .sig6(mem_sig6), .sig7(mem_sig7));
    data_handle data_handle1(.rst(rst_all), .fast_clk(fast_clk), .data(data), .wave(toggle_pos), .wave_pos(wave_pos), .loop_addr(loop_addr), .loop_repeat(loop_repeat), .increase(increase), .jump_fetch(jump_fetch), .D_jump(D_jump));
    memory_handler memory_sig0(.data(toggle_pos), .nxt_index(nxt_index0), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos0), .done(sig_done0));
    memory_handler memory_sig1(.data(mem_sig1), .nxt_index(nxt_index1), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos1), .done(sig_done1));
    memory_handler memory_sig2(.data(mem_sig2), .nxt_index(nxt_index2), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos2), .done(sig_done2));
    memory_handler memory_sig3(.data(mem_sig3), .nxt_index(nxt_index3), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos3), .done(sig_done3));
    memory_handler memory_sig4(.data(mem_sig4), .nxt_index(nxt_index4), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos4), .done(sig_done4));
    memory_handler memory_sig5(.data(mem_sig5), .nxt_index(nxt_index5), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos5), .done(sig_done5));
    memory_handler memory_sig6(.data(mem_sig6), .nxt_index(nxt_index6), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos6), .done(sig_done6));
    memory_handler memory_sig7(.data(mem_sig7), .nxt_index(nxt_index7), .fast_clk(fast_clk), .rst(rst_all), .pos(wave_pos7), .done(sig_done7));
    delay delay0(.wave_pos(wave_pos0), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig0), .delay_fetch(delay_fetch));
    delay delay1(.wave_pos(wave_pos1), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig1), .delay_fetch(delay_fetch));
    delay delay2(.wave_pos(wave_pos2), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig2), .delay_fetch(delay_fetch));
    delay delay3(.wave_pos(wave_pos3), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig3), .delay_fetch(delay_fetch));
    delay delay4(.wave_pos(wave_pos4), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig4), .delay_fetch(delay_fetch));
    delay delay5(.wave_pos(wave_pos5), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig5), .delay_fetch(delay_fetch));
    delay delay6(.wave_pos(wave_pos6), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig6), .delay_fetch(delay_fetch));
    delay delay7(.wave_pos(wave_pos7), .fast_clk(fast_clk), .main_clk(main_clk), .clk_cycle(clk_cycle), .rst(rst_all), .out_wave(sig7), .delay_fetch(delay_fetch));
    output_assembly output_assemble1(.signal0(sig0), .signal1(sig1), .signal2(sig2), .signal3(sig3), .signal4(sig4), .signal5(sig5), .signal6(sig6), .signal7(sig7), .out_wave(outwave));
    nxt_posedge edge_detector_loop (.fast_clk(fast_clk), .signal(increase), .rst(rst_all), .clk_cycle(increase_edge));
    loop loop1(.fast_clk(fast_clk), .loop_addr(loop_addr), .loop_repeat(loop_repeat), .increase(increase_edge), .rst(rst_all), .loop_fetch(loop_fetch), .L_jump(L_jump));
    
    
endmodule
