module programCounter (
    input logic rst_all,
    input logic fast_clk,
    input logic[7:0] current_address,
    output logic[7:0] P1);
    
    always @ (posedge fast_clk)
        if(rst_all)
            P1 <= 1'b0;
        else
            P1 <= current_address + 1'b1;
endmodule

module enableCtrl (
    input logic delay_fetch,
    input logic loop_fetch,
    input logic jump_fetch,
    input logic fast_clk,
    output logic enable);
    
    always_comb 
        enable = delay_fetch | jump_fetch | loop_fetch;
endmodule


module nextAddress (
    input logic rst_all,
    input logic fast_clk,
    input logic[7:0] D_jump,
    input logic[7:0] L_jump,
    //input logic fetchEnable,
    input logic delay_fetch,
    input logic loop_fetch,
    input logic jump_fetch,
    output logic[7:0] nxt_addr);
    
    logic[7:0] P1;
    logic fetchEnable;
    reg [7:0] nxt_addr_temp;
    
    enableCtrl enable1(.delay_fetch(delay_fetch), .loop_fetch(loop_fetch), .jump_fetch(jump_fetch), .enable(fetchEnable));
    programCounter programCounter1 (.rst_all(rst_all), .fast_clk(fast_clk), .current_address(nxt_addr), .P1(P1));
    
    always @ (posedge fetchEnable) begin   
        if (rst_all) begin
            nxt_addr_temp <= 1'b0;
        end else begin
            if (D_jump != 1'b0) 
                nxt_addr_temp <= D_jump;
            else if (L_jump != 1'b0)
                nxt_addr_temp <= L_jump;
            else if (P1 != 1'b0)    
                nxt_addr_temp <= P1;
            else
                nxt_addr_temp <= 1'b0;
        end
    end
    
    always @ (posedge fast_clk) begin 
        if(~rst_all) begin        
            if(fetchEnable)
                nxt_addr <= nxt_addr_temp;
            else
                nxt_addr <= nxt_addr;
         end else
            nxt_addr <= 1'b0;
    end
endmodule

module init_modules(
    input logic locked,
    output logic rst_all);
    
    always_comb begin
        if (locked)
            rst_all = 1'b0;
        else
            rst_all = 1'b1;
    end
 endmodule
 
module nxt_posedge(
    input logic fast_clk,
    input logic signal,
    input logic rst,
    output logic clk_cycle);
    
    logic sig1;
    logic sig2;
    
    DFF DFF1(.d(signal), .clk(fast_clk), .clear(rst), .q(sig1));
    DFF DFF2(.d(sig1), .clk(fast_clk), .clear(rst), .qbar(sig2));
    
    always@(posedge fast_clk) begin
        clk_cycle <= sig1 & sig2;
    end
    
endmodule 

module DFF ( 
    input logic d, 
    input logic clk, 
    input logic clear, 
    output reg q,
    output reg qbar); 
    
    always@(posedge clk) begin
        if(clear== 1) begin
            q <= 0;
            qbar <= 1;
        end else begin
            q <= d; 
            qbar <= ~d;
        end
    end 
endmodule

module clock_counter (
    input logic fast_clk,
    input logic rst,
    output logic clk_edge);
    
    reg[1:0] counter;
    always@(posedge fast_clk) begin
        if(rst) begin
            counter <= 2'b00;
            clk_edge <= 1'b0;
        end else begin
            if(counter < 2'b11) begin
                counter <= counter + 1'b1;
                clk_edge <= 1'b0;
            end else begin
                clk_edge <= 1'b1;
                counter <= 2'b00;
            end
        end 
    end
    
    
endmodule