module data_handle(
    input logic rst,
    input logic[11:0] data,
    input logic[7:0] wave_data,
    input logic fast_clk,
    output logic[7:0] wave,
    output logic[3:0] wave_repeat,
    output logic[7:0] loop_addr,
    output logic[7:0] loop_repeat,
    output logic increase,
    output logic jump_fetch,
    output logic[7:0] D_jump
    );
    
    reg [3:0] state;
    
    always_comb begin
        if(rst)
            state = 4'h0;
        else
            state = data[11:8];
    end
    
    always@(posedge fast_clk) begin 
        case (state)
        4'h0: begin
            wave = 8'h00;
            wave_repeat = 4'h0;
            loop_addr = 8'h00;
            loop_repeat = 8'h00;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = 8'h00;
            end 
        4'h2: begin
            wave = wave_data;
            wave_repeat = data[7:4];
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h4: begin
            loop_repeat = data[7:0];
            wave = wave;
            wave_repeat = wave_repeat;
            loop_addr = loop_addr;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
        4'h5: begin
            loop_addr = data[7:0];
            wave = wave;
            wave_repeat = wave_repeat;
            loop_repeat = loop_repeat;
            increase = 1'b1;
            jump_fetch = 1'b0;
            D_jump = D_jump;
            end 
         4'h6: begin
            wave = wave;
            wave_repeat = wave_repeat;
            loop_addr = loop_addr;
            loop_repeat = loop_repeat;
            increase = 1'b0;
            jump_fetch = 1'b1;
            D_jump = data[7:0];
            end 
         default: begin
            wave = 8'h00;
            wave_repeat = 4'h0;
            loop_addr = 8'h00;
            loop_repeat = 1'b0;
            increase = 1'b0;
            jump_fetch = 1'b0;
            D_jump = data[7:0];
            end 
        endcase
    end
endmodule
