module programCounter (
    input logic rst_n_all,
    input logic fast_clk,
    input logic[7:0] current_address,
    output logic[7:0] P1);
    
    always @ (posedge fast_clk)
        if(rst_n_all)
            P1 <= current_address + 1'b1;
        else
            P1 <= 1'b0;
endmodule

module enableCtrl (
    input logic delayEnable,
    input logic loopEnable,
    input logic decodeEnable,
    output logic enable);
    
    always_comb 
        enable = delayEnable | loopEnable | decodeEnable;
endmodule


module nextAddress (
    input logic rst_n_all,
    input logic fast_clk,
    input logic[7:0] D_jump,
    input logic[7:0] L_jump,
    input logic fetchEnable,
    output logic[7:0] nxt_addr);
    
    logic[7:0] P1;
    logic[7:0] nxt_addr_temp;
    
    programCounter programCounter1 (.rst_n_all(rst_n_all), .fast_clk(fast_clk), .current_address(nxt_addr), .P1(P1));
    
    always @ (posedge fetchEnable) begin   
        if (rst_n_all) begin
            if (D_jump != 1'b0) 
                nxt_addr_temp <= D_jump;
            else if (L_jump != 1'b0)
                nxt_addr_temp <= L_jump;
            else if (P1 != 1'b0)    
                nxt_addr_temp <= P1;
            else
                nxt_addr_temp <= 1'b0;
         end else
            nxt_addr_temp <= 1'b0;
    end
    
    always @ (posedge fast_clk) begin 
        if(rst_n_all) begin        
            if(fetchEnable)
                nxt_addr <= nxt_addr_temp;
            else
                nxt_addr <= nxt_addr;
         end else
            nxt_addr <= 1'b0;
    end
endmodule

module init_modules(
    input logic locked,
    output logic rst_n_all);
    
    always_comb begin
        if (locked)
            rst_n_all = 1'b1;
        else
            rst_n_all = 1'b0;
    end
 endmodule

module nxt_posedge(
    input logic fast_clk,
    input logic signal,
    input logic rst,
    output logic clk_cycle);
    
    logic sig1;
    logic sig2;
    
    DFF DFF1(.d(signal), .clk(fast_clk), .clear(rst), .q(sig1));
    DFF DFF2(.d(sig1), .clk(fast_clk), .clear(rst), .qbar(sig2));
    
    always@(posedge fast_clk) begin
        clk_cycle <= sig1 & sig2;
    end
    
endmodule 

module DFF ( 
    input logic d, 
    input logic clk, 
    input logic clear, 
    output reg q,
    output reg qbar); 
    
    always@(posedge clk) begin
        if(clear== 1) begin
            q <= 0;
            qbar <= 1;
        end else begin
            q <= d; 
            qbar <= ~d;
        end
    end 
endmodule


