module decode(
    input logic[11:0] data,
    input logic[7:0] wave_data,
    input logic rst_n,
    output logic[7:0] wave,
    output logic[3:0] hold,
    output logic[7:0] L_Address,
    output logic[7:0] itterate,
    output logic increase,
    output logic[7:0] D_jump,
    output logic fetchEnable);
    
    
    always_comb begin
        if (!rst_n) begin
            wave <= 1'b0;
            hold <= 1'b0;
            L_Address <= 1'b0;
            itterate <= 1'b0;
            increase <= 1'b0;
            D_jump <= 1'b0;
            fetchEnable <= 1'b0;
        end else if (data[11:8] == 4'b0010) begin                    //normal wave
            wave <= wave_data;
            hold <= data[7:4];
            L_Address <= L_Address;
            itterate <= itterate;
            increase <= 1'b0;
            D_jump <= 1'b0;
            fetchEnable <= 1'b0;
        end else if (data[11:8] == 4'b0110) begin           //D-Jump instruction continue loop
            wave <= wave;
            hold <= hold;
            L_Address <= L_Address;
            itterate <= itterate;
            increase <= 1'b0;
            D_jump <= data[7:0];
            fetchEnable <= 1'b1;
        end else if (data[11:8] == 4'b0111) begin           //D-Jump instruction break loop
            wave <= wave;
            hold <= hold;
            L_Address <= L_Address;
            itterate <= 1'b0;
            increase <= 1'b0;
            D_jump <= data[7:0];
            fetchEnable <= 1'b1;
        end else if (data[11:8] == 4'b0100) begin          //L-start
            wave <= wave;
            hold <= hold;
            L_Address <= L_Address;
            itterate <= data[7:0];
            increase <= 1'b0;
            D_jump <= 1'b0;
            fetchEnable <= 1'b1;    
        end else if (data[11:8] == 4'b0101) begin          //L-stop
            wave <= wave;
            hold <= hold;
            L_Address <= data[7:0];
            itterate <= itterate;
            increase <= 1'b1;
            D_jump <= 1'b0;
            fetchEnable <= 1'b0;      
        end else begin
            wave <= wave;
            hold <= hold;
            L_Address <= 1'b0;
            itterate <= itterate;
            increase <= 1'b0;
            D_jump <= 1'b0;
            fetchEnable <= 1'b0;
        end
    end
endmodule