module counter #(
    parameter SIZE = 4
)(
    input logic clk,
    input logic rst,
    output logic[SIZE-1:0] count
);

    always @ (posedge clk) begin
        if (rst) begin
            count <= 'b0;
        end
        else begin
            count <= count + 1'b1;
        end
    end
endmodule 

module delay(
    input logic[7:0] wave,
    input logic[3:0] hold,
    input logic clk,
    input logic fast_clk,
    input logic rst_n_all,
    output logic[7:0] outwave,
    output logic fetchEnable);
    
    logic [3:0] delaycount;
    logic [3:0] currentHold;
    logic [7:0] currentOut;
    logic [7:0] next_outwave;
    logic rst;
    logic changed;
    
    small_counter delayCounter2 (.CLK(clk), .SCLR(rst), .Q(delaycount));
   
    //counter delayCounter(.clk(clk), .rst(rst), .count(delaycount));
    
    always @ (posedge fast_clk) begin
        if (rst_n_all) begin 
            if(delaycount == 0 & currentHold == 0) begin        //initial start
                next_outwave <= wave;
                rst <= 1'b1;
                fetchEnable <= 1'b1;
                currentOut <= wave;
                currentHold <= hold;
                changed <= 1'b1;
            end
            else if ((delaycount+1) == currentHold & ~changed) begin 
                currentOut <= wave;             //load new wave to output
                currentHold <= hold;            //load new hold count to register
                rst <= 1'b1;                  //reset counter to zero
                fetchEnable <= 1'b1;            //signal to start fetching next wave
                next_outwave <= wave;
                changed <= 1'b1;
            end 
            else if ((delaycount+1) == currentHold & changed) begin
                currentOut <= currentOut;             //load new wave to output
                currentHold <= currentHold;            //load new hold count to register
                rst <= 1'b1;                  //reset counter to zero
                fetchEnable <= 1'b0;            //signal to start fetching next wave
                next_outwave <= next_outwave;
                changed <= 1'b1;
            end
            else if (delaycount == 0 & currentHold != 0) begin        //make sure reset is read
                next_outwave <= currentOut;
                rst <= 1'b0;
                fetchEnable <= 1'b0;
                currentOut <= currentOut;
                currentHold <= currentHold;
                changed <= 1'b0;
            end
            else begin
                next_outwave <= currentOut;
                rst <= rst;
                fetchEnable <= 1'b0;
                currentOut <= currentOut;
                currentHold <= currentHold;  
                changed <= 1'b0;
            end 
        end else begin
            rst <= 1'b1;
            next_outwave <= 8'h00;
            fetchEnable <= 1'b0;
            currentOut <= wave;
            currentHold <= hold;  
            changed <= 1'b0;
        end       
    end
    
    always @ (posedge clk) begin
        outwave <= next_outwave;
    end
endmodule


module loop(
    input logic[7:0] address,
    input logic[7:0] itterate,
    input logic increase,
    input logic fast_clk,
    input logic rst_n_all,
    output logic[7:0] L_jump,
    output logic fetchEnable);
    
    //logic rst_n;
    logic rst;
    logic[7:0] loopCount;
    logic[7:0] L_addr;
    logic enable;
    logic[1:0] count;
    
    //counter # (.SIZE(8)) loopcounter(.clk(increase), .rst_n(rst_n), .count(loopCount));
    large_counter loopCounter (.CLK(increase), .SCLR(rst), .Q(loopCount));

    always @(increase) begin
        if(rst_n_all & increase) begin
            if (loopCount == itterate-1) begin
                //rst_n <= 1'b0;
                rst <= 1'b1;
                L_addr <= 1'b0;
                enable <= 1'b1;
                count <= 1'b0;
            end else begin 
                //rst_n <= 1'b1;
                rst <= 1'b0;
                L_addr <= address;
                enable <= 1'b1;
                count <= 1'b0;
            end
        end else if (rst_n_all) begin
            enable <= 1'b0;
            L_addr <= 1'b0;
            //rst_n <= 1'b1;
            rst <= 1'b0;
            count <= 1'b0;
        end else begin
            //rst_n <= 1'b0;
            rst <= 1'b1;
            L_addr <= 1'b0;
            enable <= 1'b0;
            count <= 1'b0;
        end 
    end
    
    always @ (posedge fast_clk) begin
        L_jump <= L_addr;
        if(count > 1'b0)
            fetchEnable <= 1'b0;
        else
            fetchEnable <= enable;
        count <= count + 1'b1;
    end
endmodule
        
        

